"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


CVVResponseCode = Union[
    Literal["match", "no_match", "unavailable", "not_provided"], UnrecognizedStr
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy import utils
from gr4vy.models import errordetail as models_errordetail
from gr4vy.types import BaseModel
from gr4vy.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional
from typing_extensions import Annotated


class Error502Data(BaseModel):
    TYPE: Annotated[
        Annotated[Optional[Literal["error"]], AfterValidator(validate_const("error"))],
        pydantic.Field(alias="type"),
    ] = "error"
    r"""Always `error`."""

    code: Optional[str] = "bad_gateway"
    r"""Always `bad_gateway`"""

    status: Optional[int] = 502
    r"""Always `502`."""

    message: Optional[str] = "Request could not be processed"
    r"""A human readable message that provides more context to the error."""

    details: Optional[List[models_errordetail.ErrorDetail]] = None
    r"""A list of details that further ellaborate on the error."""


class Error502(Exception):
    data: Error502Data

    def __init__(self, data: Error502Data):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, Error502Data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


ReportExecutionStatus = Union[
    Literal["dispatched", "failed", "pending", "processing", "succeeded"],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .googlepayassurancedetails import (
    GooglePayAssuranceDetails,
    GooglePayAssuranceDetailsTypedDict,
)
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Any, Dict, Literal, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


TokenTypedDict = TypeAliasType("TokenTypedDict", Union[str, Dict[str, Any]])
r"""The opaque token as received from the Google Pay JS library. This format may change between JS library versions."""


Token = TypeAliasType("Token", Union[str, Dict[str, Any]])
r"""The opaque token as received from the Google Pay JS library. This format may change between JS library versions."""


class GooglePayPaymentMethodCreateTypedDict(TypedDict):
    r"""Create a Google Pay transaction with a device token."""

    token: TokenTypedDict
    r"""The opaque token as received from the Google Pay JS library. This format may change between JS library versions."""
    buyer_external_identifier: NotRequired[Nullable[str]]
    r"""The external identifier of the buyer to create a payment for."""
    buyer_id: NotRequired[Nullable[str]]
    r"""The ID of the buyer to retrieve billing details for."""
    cardholder_name: NotRequired[Nullable[str]]
    r"""The card holder name associated to the original card for the token."""
    redirect_url: NotRequired[Nullable[str]]
    r"""The URL to redirect a user back to after the complete 3DS in browser."""
    card_suffix: NotRequired[Nullable[str]]
    r"""The last 4 digits of the original card used to generate the token."""
    card_scheme: NotRequired[Nullable[str]]
    r"""The original card scheme for which the token was generated."""
    card_type: NotRequired[Nullable[str]]
    r"""The payment scheme of the card."""
    method: Literal["googlepay"]
    r"""Always `googlepay`"""
    assurance_details: NotRequired[Nullable[GooglePayAssuranceDetailsTypedDict]]
    r"""The assurance details provided by Google Pay"""


class GooglePayPaymentMethodCreate(BaseModel):
    r"""Create a Google Pay transaction with a device token."""

    token: Token
    r"""The opaque token as received from the Google Pay JS library. This format may change between JS library versions."""

    buyer_external_identifier: OptionalNullable[str] = UNSET
    r"""The external identifier of the buyer to create a payment for."""

    buyer_id: OptionalNullable[str] = UNSET
    r"""The ID of the buyer to retrieve billing details for."""

    cardholder_name: OptionalNullable[str] = UNSET
    r"""The card holder name associated to the original card for the token."""

    redirect_url: OptionalNullable[str] = UNSET
    r"""The URL to redirect a user back to after the complete 3DS in browser."""

    card_suffix: OptionalNullable[str] = UNSET
    r"""The last 4 digits of the original card used to generate the token."""

    card_scheme: OptionalNullable[str] = UNSET
    r"""The original card scheme for which the token was generated."""

    card_type: OptionalNullable[str] = UNSET
    r"""The payment scheme of the card."""

    METHOD: Annotated[
        Annotated[Literal["googlepay"], AfterValidator(validate_const("googlepay"))],
        pydantic.Field(alias="method"),
    ] = "googlepay"
    r"""Always `googlepay`"""

    assurance_details: OptionalNullable[GooglePayAssuranceDetails] = UNSET
    r"""The assurance details provided by Google Pay"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "buyer_external_identifier",
            "buyer_id",
            "cardholder_name",
            "redirect_url",
            "card_suffix",
            "card_scheme",
            "card_type",
            "assurance_details",
        ]
        nullable_fields = [
            "buyer_external_identifier",
            "buyer_id",
            "cardholder_name",
            "redirect_url",
            "card_suffix",
            "card_scheme",
            "card_type",
            "assurance_details",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from gr4vy.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, TypedDict


class AccountUpdaterInquirySummaryTypedDict(TypedDict):
    id: str
    r"""The ID for the account updater inquiry."""
    payment_method_id: str
    r"""The ID of the payment method"""
    type: Literal["account-updater-inquiry"]
    r"""Always `account-updater-inquiry`"""


class AccountUpdaterInquirySummary(BaseModel):
    id: str
    r"""The ID for the account updater inquiry."""

    payment_method_id: str
    r"""The ID of the payment method"""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["account-updater-inquiry"]],
            AfterValidator(validate_const("account-updater-inquiry")),
        ],
        pydantic.Field(alias="type"),
    ] = "account-updater-inquiry"
    r"""Always `account-updater-inquiry`"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


ThreeDSecureStatus = Union[
    Literal["setup_error", "error", "declined", "cancelled", "complete"],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


ProductType = Union[
    Literal[
        "physical",
        "discount",
        "shipping_fee",
        "sales_tax",
        "digital",
        "gift_card",
        "store_credit",
        "surcharge",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cartitem import CartItem, CartItemTypedDict
from .guestbuyer_input import GuestBuyerInput, GuestBuyerInputTypedDict
from .statementdescriptor import StatementDescriptor, StatementDescriptorTypedDict
from .transactionintent import TransactionIntent
from .transactionpaymentsource import TransactionPaymentSource
from datetime import datetime
from gr4vy.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from gr4vy.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


Locale = Union[Literal["en", "en-GB", "pt", "pt-BR", "es"], UnrecognizedStr]


class PaymentLinkCreateTypedDict(TypedDict):
    amount: int
    r"""The amount for the payment link."""
    country: str
    r"""The country code for the payment link."""
    currency: str
    r"""The currency code for the payment link."""
    buyer: NotRequired[Nullable[GuestBuyerInputTypedDict]]
    r"""The guest buyer for the payment link."""
    expires_at: NotRequired[Nullable[datetime]]
    r"""The expiration date and time for the payment link."""
    connection_options: NotRequired[Nullable[Dict[str, Dict[str, Any]]]]
    r"""Connection options for the payment link."""
    external_identifier: NotRequired[Nullable[str]]
    r"""The merchant reference for the payment link."""
    statement_descriptor: NotRequired[Nullable[StatementDescriptorTypedDict]]
    r"""The statement descriptor for the payment link."""
    locale: NotRequired[Nullable[Locale]]
    r"""The locale for the payment link."""
    merchant_name: NotRequired[Nullable[str]]
    r"""The merchant's display name."""
    merchant_url: NotRequired[Nullable[str]]
    r"""The merchant's website URL."""
    merchant_banner_url: NotRequired[Nullable[str]]
    r"""The merchant's banner image URL."""
    merchant_color: NotRequired[Nullable[str]]
    r"""The merchant's brand color."""
    merchant_message: NotRequired[Nullable[str]]
    r"""A message from the merchant."""
    merchant_terms_and_conditions_url: NotRequired[Nullable[str]]
    r"""URL to the merchant's terms and conditions."""
    merchant_favicon_url: NotRequired[Nullable[str]]
    r"""URL to the merchant's favicon."""
    intent: NotRequired[TransactionIntent]
    return_url: NotRequired[Nullable[str]]
    r"""The return URL after payment completion."""
    cart_items: NotRequired[Nullable[List[CartItemTypedDict]]]
    r"""The cart items for the payment link."""
    metadata: NotRequired[Nullable[Dict[str, Any]]]
    r"""Arbitrary metadata for the payment link."""
    payment_source: NotRequired[TransactionPaymentSource]
    r"""The way payment method information made it to this transaction."""


class PaymentLinkCreate(BaseModel):
    amount: int
    r"""The amount for the payment link."""

    country: str
    r"""The country code for the payment link."""

    currency: str
    r"""The currency code for the payment link."""

    buyer: OptionalNullable[GuestBuyerInput] = UNSET
    r"""The guest buyer for the payment link."""

    expires_at: OptionalNullable[datetime] = UNSET
    r"""The expiration date and time for the payment link."""

    connection_options: OptionalNullable[Dict[str, Dict[str, Any]]] = UNSET
    r"""Connection options for the payment link."""

    external_identifier: OptionalNullable[str] = UNSET
    r"""The merchant reference for the payment link."""

    statement_descriptor: OptionalNullable[StatementDescriptor] = UNSET
    r"""The statement descriptor for the payment link."""

    locale: Annotated[
        OptionalNullable[Locale], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""The locale for the payment link."""

    merchant_name: OptionalNullable[str] = UNSET
    r"""The merchant's display name."""

    merchant_url: OptionalNullable[str] = UNSET
    r"""The merchant's website URL."""

    merchant_banner_url: OptionalNullable[str] = UNSET
    r"""The merchant's banner image URL."""

    merchant_color: OptionalNullable[str] = UNSET
    r"""The merchant's brand color."""

    merchant_message: OptionalNullable[str] = UNSET
    r"""A message from the merchant."""

    merchant_terms_and_conditions_url: OptionalNullable[str] = UNSET
    r"""URL to the merchant's terms and conditions."""

    merchant_favicon_url: OptionalNullable[str] = UNSET
    r"""URL to the merchant's favicon."""

    intent: Annotated[
        Optional[TransactionIntent], PlainValidator(validate_open_enum(False))
    ] = None

    return_url: OptionalNullable[str] = UNSET
    r"""The return URL after payment completion."""

    cart_items: OptionalNullable[List[CartItem]] = UNSET
    r"""The cart items for the payment link."""

    metadata: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Arbitrary metadata for the payment link."""

    payment_source: Annotated[
        Optional[TransactionPaymentSource], PlainValidator(validate_open_enum(False))
    ] = None
    r"""The way payment method information made it to this transaction."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "buyer",
            "expires_at",
            "connection_options",
            "external_identifier",
            "statement_descriptor",
            "locale",
            "merchant_name",
            "merchant_url",
            "merchant_banner_url",
            "merchant_color",
            "merchant_message",
            "merchant_terms_and_conditions_url",
            "merchant_favicon_url",
            "intent",
            "return_url",
            "cart_items",
            "metadata",
            "payment_source",
        ]
        nullable_fields = [
            "buyer",
            "expires_at",
            "connection_options",
            "external_identifier",
            "statement_descriptor",
            "locale",
            "merchant_name",
            "merchant_url",
            "merchant_banner_url",
            "merchant_color",
            "merchant_message",
            "merchant_terms_and_conditions_url",
            "merchant_favicon_url",
            "return_url",
            "cart_items",
            "metadata",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


TransactionStatus = Union[
    Literal[
        "processing",
        "authorization_succeeded",
        "authorization_declined",
        "authorization_failed",
        "authorization_voided",
        "authorization_void_pending",
        "capture_succeeded",
        "capture_pending",
        "buyer_approval_pending",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .creator import Creator, CreatorTypedDict
from .refundstatus import RefundStatus
from .refundtargettype import RefundTargetType
from datetime import datetime
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const, validate_open_enum
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class RefundTypedDict(TypedDict):
    id: str
    r"""The ID for the refund."""
    transaction_id: str
    r"""The ID of the transaction associated with this refund."""
    status: RefundStatus
    currency: str
    r"""The currency code for this refund.  Will always match that of the associated transaction."""
    amount: int
    r"""The amount requested for this refund."""
    target_type: RefundTargetType
    reconciliation_id: str
    r"""The base62 encoded refund ID. This represents a shorter version of this refund's `id` which is sent to payment services, anti-fraud services, and other connectors. You can use this ID to reconcile a payment service's refund against our system."""
    transaction_reconciliation_id: str
    r"""The base62 encoded transaction ID. This represents a shorter version of the related transaction's `id` which is sent to payment services, anti-fraud services, and other connectors. You can use this ID to reconcile a payment service's transaction against our system."""
    created_at: datetime
    r"""The date this refund was created at."""
    updated_at: datetime
    r"""The date this refund was last updated at."""
    type: Literal["refund"]
    r"""Always `refund`."""
    payment_service_refund_id: NotRequired[Nullable[str]]
    r"""The payment service's unique ID for the refund."""
    reason: NotRequired[Nullable[str]]
    r"""The reason for this refund. Could be a multiline string."""
    target_id: NotRequired[Nullable[str]]
    r"""The optional ID of the instrument that was refunded. This may be `null` if the instrument was not stored."""
    external_identifier: NotRequired[Nullable[str]]
    r"""An external identifier that can be used to match the refund against your own records."""
    transaction_external_identifier: NotRequired[Nullable[str]]
    r"""An external identifier that can be used to match the transaction against your own records."""
    creator: NotRequired[Nullable[CreatorTypedDict]]
    r"""The user that created this resource"""


class Refund(BaseModel):
    id: str
    r"""The ID for the refund."""

    transaction_id: str
    r"""The ID of the transaction associated with this refund."""

    status: Annotated[RefundStatus, PlainValidator(validate_open_enum(False))]

    currency: str
    r"""The currency code for this refund.  Will always match that of the associated transaction."""

    amount: int
    r"""The amount requested for this refund."""

    target_type: Annotated[RefundTargetType, PlainValidator(validate_open_enum(False))]

    reconciliation_id: str
    r"""The base62 encoded refund ID. This represents a shorter version of this refund's `id` which is sent to payment services, anti-fraud services, and other connectors. You can use this ID to reconcile a payment service's refund against our system."""

    transaction_reconciliation_id: str
    r"""The base62 encoded transaction ID. This represents a shorter version of the related transaction's `id` which is sent to payment services, anti-fraud services, and other connectors. You can use this ID to reconcile a payment service's transaction against our system."""

    created_at: datetime
    r"""The date this refund was created at."""

    updated_at: datetime
    r"""The date this refund was last updated at."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["refund"]], AfterValidator(validate_const("refund"))
        ],
        pydantic.Field(alias="type"),
    ] = "refund"
    r"""Always `refund`."""

    payment_service_refund_id: OptionalNullable[str] = UNSET
    r"""The payment service's unique ID for the refund."""

    reason: OptionalNullable[str] = UNSET
    r"""The reason for this refund. Could be a multiline string."""

    target_id: OptionalNullable[str] = UNSET
    r"""The optional ID of the instrument that was refunded. This may be `null` if the instrument was not stored."""

    external_identifier: OptionalNullable[str] = UNSET
    r"""An external identifier that can be used to match the refund against your own records."""

    transaction_external_identifier: OptionalNullable[str] = UNSET
    r"""An external identifier that can be used to match the transaction against your own records."""

    creator: OptionalNullable[Creator] = UNSET
    r"""The user that created this resource"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "type",
            "payment_service_refund_id",
            "reason",
            "target_id",
            "external_identifier",
            "transaction_external_identifier",
            "creator",
        ]
        nullable_fields = [
            "payment_service_refund_id",
            "reason",
            "target_id",
            "external_identifier",
            "transaction_external_identifier",
            "creator",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .field import FieldT, FieldTTypedDict
from .merchantprofileschemesummary import (
    MerchantProfileSchemeSummary,
    MerchantProfileSchemeSummaryTypedDict,
)
from .method import Method
from .paymentservicestatus import PaymentServiceStatus
from datetime import datetime
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const, validate_open_enum
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Dict, List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PaymentServiceTypedDict(TypedDict):
    merchant_account_id: str
    r"""The ID of the merchant account this job belongs to."""
    payment_service_definition_id: str
    r"""The definition ID of the service that has been configured."""
    method: Method
    display_name: str
    r"""The display name for the payment service."""
    position: int
    r"""Deprecated field used to define the order in which to process payment services"""
    accepted_currencies: List[str]
    r"""A list of currencies for which this service is enabled, in ISO 4217 three-letter code format."""
    accepted_countries: List[str]
    r"""A list of countries for which this service is enabled, in ISO two-letter code format."""
    payment_method_tokenization_enabled: bool
    r"""Defines if this payment service support payment method tokenization."""
    network_tokens_enabled: bool
    r"""Defines if this payment service supports network tokens."""
    open_loop: bool
    r"""Defines if this payment service is open loop."""
    settlement_reporting_enabled: bool
    r"""Defines if this payment service has settlement reporting enabled."""
    created_at: datetime
    r"""The date and time when this payment service was first created in our system."""
    updated_at: datetime
    r"""The date and time when this payment service was last updated in our system."""
    type: Literal["payment-service"]
    r"""Always `payment-service`"""
    id: NotRequired[str]
    active: NotRequired[bool]
    r"""Defines if this payment service is currently active."""
    status: NotRequired[PaymentServiceStatus]
    three_d_secure_enabled: NotRequired[Nullable[bool]]
    r"""Defines if this payment service has 3DS enabled."""
    merchant_profile: NotRequired[
        Nullable[Dict[str, Nullable[MerchantProfileSchemeSummaryTypedDict]]]
    ]
    r"""An object containing a key for each supported card schemes, and for each key an object with the 3DS profile for this service for that scheme."""
    webhook_url: NotRequired[Nullable[str]]
    r"""The URL that needs to be configured with this payment service as the receiving endpoint for webhooks from the service to our system. Currently, we dp not yet automatically register webhooks on setup, and therefore webhooks need to be registered manually by the merchant."""
    fields: NotRequired[Nullable[List[FieldTTypedDict]]]
    r"""The non-secret credential fields that have been configured for this payment service. Any secret fields are omitted."""
    reporting_fields: NotRequired[Nullable[List[FieldTTypedDict]]]
    r"""The non-secret reporting fields that have been configured for this payment service. Any secret fields are omitted."""
    is_deleted: NotRequired[bool]
    r"""Defines if this payment service has been deleted"""


class PaymentService(BaseModel):
    merchant_account_id: str
    r"""The ID of the merchant account this job belongs to."""

    payment_service_definition_id: str
    r"""The definition ID of the service that has been configured."""

    method: Annotated[Method, PlainValidator(validate_open_enum(False))]

    display_name: str
    r"""The display name for the payment service."""

    position: int
    r"""Deprecated field used to define the order in which to process payment services"""

    accepted_currencies: List[str]
    r"""A list of currencies for which this service is enabled, in ISO 4217 three-letter code format."""

    accepted_countries: List[str]
    r"""A list of countries for which this service is enabled, in ISO two-letter code format."""

    payment_method_tokenization_enabled: bool
    r"""Defines if this payment service support payment method tokenization."""

    network_tokens_enabled: bool
    r"""Defines if this payment service supports network tokens."""

    open_loop: bool
    r"""Defines if this payment service is open loop."""

    settlement_reporting_enabled: bool
    r"""Defines if this payment service has settlement reporting enabled."""

    created_at: datetime
    r"""The date and time when this payment service was first created in our system."""

    updated_at: datetime
    r"""The date and time when this payment service was last updated in our system."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["payment-service"]],
            AfterValidator(validate_const("payment-service")),
        ],
        pydantic.Field(alias="type"),
    ] = "payment-service"
    r"""Always `payment-service`"""

    id: Optional[str] = "The ID of the payment service"

    active: Optional[bool] = True
    r"""Defines if this payment service is currently active."""

    status: Annotated[
        Optional[PaymentServiceStatus], PlainValidator(validate_open_enum(False))
    ] = None

    three_d_secure_enabled: OptionalNullable[bool] = UNSET
    r"""Defines if this payment service has 3DS enabled."""

    merchant_profile: OptionalNullable[
        Dict[str, Nullable[MerchantProfileSchemeSummary]]
    ] = UNSET
    r"""An object containing a key for each supported card schemes, and for each key an object with the 3DS profile for this service for that scheme."""

    webhook_url: OptionalNullable[str] = UNSET
    r"""The URL that needs to be configured with this payment service as the receiving endpoint for webhooks from the service to our system. Currently, we dp not yet automatically register webhooks on setup, and therefore webhooks need to be registered manually by the merchant."""

    fields: OptionalNullable[List[FieldT]] = UNSET
    r"""The non-secret credential fields that have been configured for this payment service. Any secret fields are omitted."""

    reporting_fields: OptionalNullable[List[FieldT]] = UNSET
    r"""The non-secret reporting fields that have been configured for this payment service. Any secret fields are omitted."""

    is_deleted: Optional[bool] = False
    r"""Defines if this payment service has been deleted"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "type",
            "id",
            "active",
            "status",
            "three_d_secure_enabled",
            "merchant_profile",
            "webhook_url",
            "fields",
            "reporting_fields",
            "is_deleted",
        ]
        nullable_fields = [
            "three_d_secure_enabled",
            "merchant_profile",
            "webhook_url",
            "fields",
            "reporting_fields",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class GiftCardTokenTransactionCreateTypedDict(TypedDict):
    id: str
    r"""The ID for the gift card to charge."""
    amount: int
    r"""The monetary amount for this transaction to charge against the gift card, in the smallest currency unit for the given currency, for example `1299` cents to create an authorization for `$12.99`."""


class GiftCardTokenTransactionCreate(BaseModel):
    id: str
    r"""The ID for the gift card to charge."""

    amount: int
    r"""The monetary amount for this transaction to charge against the gift card, in the smallest currency unit for the given currency, for example `1299` cents to create an authorization for `$12.99`."""
